// IECachePurgeGUI.cpp - Partie interface graphique de purge le cache Internet Explorer 5.x, 6.0 de tous les fichiers inutiles.
// Copyright (c) 2001, 2005, Thierry MARTIN.
// Version #1.10.
// Cre le jeudi 1 juin 2005.

// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon
// les termes de la Licence Publique Gnrale GNU publie par la Free Software
// Foundation (version 2 ou bien toute autre version ultrieure choisie par
// vous).
//
// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
// GARANTIE, ni explicite ni implicite, y compris les garanties de
// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
// Licence Publique Gnrale GNU pour plus de dtails.
//
// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme
// temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
// tats-Unis.

// Vous pouvez me joindre par e-mail  : iecachepurge@martin-thierry.nom.fr

// Versions : voir le fichier IECachePurge.h

#include <wx/wx.h>
#include <wx/progdlg.h>
#include <wx/imaglist.h>
#include <wx/treectrl.h>
#include <wx/file.h>
#include <wx/cmdline.h>

#include "IECachePurge.h"

// Dclaration des classes

class IECPApp : public wxApp
{
private:
	bool m_purgeAuto;
public:
	virtual bool OnCmdLineParsed (wxCmdLineParser& parser_);
	virtual bool OnInit ();
	virtual void OnInitCmdLine (wxCmdLineParser& parser_);
	bool PurgeAuto ();
	IECPApp ();
};

class IECPTreeCtrl : public wxTreeCtrl, public TCacheIEAPurger
{
public:
	enum
	{
		Image_Dossier,
		Image_DossierOuvert,
		Image_Document
	};

	IECPTreeCtrl (wxWindow* parent_, const wxPoint& position_, const wxSize taille_);
	virtual void AjoutDossier (const wxString& nom_);
	virtual void AjoutFichier (const wxString& nom_);
	void DeveloppeBase ();
	void OnCacheSupprimer (wxCommandEvent& evenement);
	void OnCacheVoir (wxCommandEvent& evenement);
	virtual wxString PremierFichier ();
	virtual bool SupprimeFichier (const wxString& nom_);
private:
	void OnClicDroit (wxMouseEvent& event_);

	wxTreeItemId m_baseId;
	wxString m_fichierClicNom;
	wxTreeItemId m_fichierClicId;
	wxTreeItemId m_dossierId;

	DECLARE_EVENT_TABLE()
};

class IECPFrame : public wxFrame
{
public:
	IECPFrame (const wxString& title_, const wxPoint& pos_, const wxSize& size_);

	void OnAPropos (wxCommandEvent& event_);
	void OnPurge (wxCommandEvent& event_);
	void OnRecherche (wxCommandEvent& event_);
	void OnSortie (wxCommandEvent& event_);
private:
	IECPTreeCtrl*		m_treeCtrl;
	TEnsembleCacheIE* m_ensembleCacheIE;
	DECLARE_EVENT_TABLE();

	void Purge ();
};

enum
{
	IECPurge_Sortie = 1,
	IECPurge_Recherche,
	IECPurge_Purge,
	IECPurge_APropos,
	IECPurge_CacheSupprimer,
	IECPurge_CacheVoir
};

// Dfinition des constantes globales

static const char titreApplication [] = "Internet Explorer Cache Purge";

// Dfinition des fonctions membres  l'cran

BEGIN_EVENT_TABLE(IECPFrame, wxFrame)
	EVT_MENU(IECPurge_Sortie,	  IECPFrame::OnSortie)
	EVT_MENU(IECPurge_Recherche, IECPFrame::OnRecherche)
	EVT_MENU(IECPurge_Purge,	  IECPFrame::OnPurge)
	EVT_MENU(IECPurge_APropos,	  IECPFrame::OnAPropos)
END_EVENT_TABLE()

IECPFrame::IECPFrame (const wxString& title_, const wxPoint& pos_,
							 const wxSize& size_)
 : wxFrame (NULL, -1, title_, pos_, size_)
{
	m_treeCtrl = 0;
	SetIcon (wxICON(iecachepurge));

	// Menu
	wxMenu* menuFichier = new wxMenu ();
	wxMenu* menuAide = new wxMenu ();

	menuFichier->Append (IECPurge_Recherche, "&Rechercher\tCtrl-R", "Commence la recherche des fichiers inutiles.");
	menuFichier->Append (IECPurge_Purge, "&Purger...\tCtrl-P", "Supprime les fichiers inutiles.");
	menuFichier->AppendSeparator ();
	menuFichier->Append (IECPurge_Sortie, "&Quitter\tAlt-Q", "Quitter IECachePurge.");
	menuAide->Append (IECPurge_APropos, "& propos", " propos d'Internet Explorer Cache Purge.");

	wxMenuBar* menuBar = new wxMenuBar ();
	menuBar->Append (menuFichier, "&Fichier");
	menuBar->Append (menuAide, "&Aide");

	SetMenuBar (menuBar);

	menuBar->Enable (IECPurge_Purge, FALSE);

	// Barre d'tat
	CreateStatusBar (1);
	SetStatusText ("Commencer par rechercher les fichiers inutiles (Fichier/Rechercher).");
}

void IECPFrame::OnSortie (wxCommandEvent& event_)
{
	Close ();
}

void IECPFrame::OnRecherche (wxCommandEvent& event_)
{
	SetStatusText (wxEmptyString);
	GetMenuBar ()->Enable (IECPurge_Purge, FALSE);
	if (m_ensembleCacheIE)
	{
		delete m_ensembleCacheIE;
		m_ensembleCacheIE = NULL;
	}
	if (m_treeCtrl)
	{
		m_treeCtrl->Destroy ();
		m_treeCtrl = NULL;
	}

	m_treeCtrl = new IECPTreeCtrl (this, GetClientAreaOrigin (), GetClientSize ());
	m_ensembleCacheIE = new TEnsembleCacheIE (*m_treeCtrl);
	if (!m_ensembleCacheIE->Erreur ().IsEmpty ())
	{
		wxMessageBox (m_ensembleCacheIE->Erreur (), titreApplication, wxOK | wxICON_EXCLAMATION, this);
		return;
	}
	wxProgressDialog* progress = new wxProgressDialog ("Recherche des fichiers inutiles",
																		"Parcours du cache Internet Explorer", 0, this,
																		wxPD_APP_MODAL | wxPD_CAN_ABORT);
	while (m_ensembleCacheIE->RechercheSuivant ())
	{
		if (!progress->Update (-1))
		{
			progress->Show (FALSE);
			progress->Destroy ();
			m_treeCtrl->Destroy ();
			m_treeCtrl = NULL;
			return;
		}
	}
	progress->Show (FALSE);
	progress->Destroy ();
	if (!m_ensembleCacheIE->Erreur ().IsEmpty ())
	{
		wxMessageBox (m_ensembleCacheIE->Erreur (), titreApplication, wxOK | wxICON_EXCLAMATION, this);
		return;
	}

	wxString etat;

	if (!m_ensembleCacheIE->NombreFichiers ())
	{
		delete m_ensembleCacheIE; m_ensembleCacheIE = NULL;
		m_treeCtrl->Destroy (); m_treeCtrl = 0;
		etat = "Il n'y a pas de fichier  purger.";
		wxMessageBox (etat, titreApplication, wxICON_INFORMATION | wxOK, this);
	}
	else
	{
		m_treeCtrl->DeveloppeBase ();
		m_treeCtrl->Show (TRUE);
		GetMenuBar ()->Enable (IECPurge_Purge, TRUE);
		etat = wxString::Format ("%u fichier", m_ensembleCacheIE->NombreFichiers ());
		if (m_ensembleCacheIE->NombreFichiers () > 1) etat += "s";
		etat += wxString::Format (" pour %lld octet", m_ensembleCacheIE->TailleTotale ());
		if (m_ensembleCacheIE->TailleTotale () > 1)
			etat += "s, ils peuvent tre purgs";
		else
			etat += ", il peut tre purg";
		etat += " (Fichier/Purger).";
	}
	SetStatusText (etat);
}

void IECPFrame::OnPurge (wxCommandEvent& event_)
{
	if (wxMessageBox ("Suppression de tous les fichiers inutiles ?", titreApplication, wxICON_QUESTION | wxYES_NO, this) ==
		 wxYES)
	{
		SetStatusText (wxEmptyString);
		if (m_ensembleCacheIE && m_treeCtrl)
		{
			Purge ();
		}
		else
		{
			wxMessageBox ("Avant de purger les fichiers il faut lancer la recherche.", titreApplication, wxICON_EXCLAMATION | wxOK, this);
		}
	}
}

void IECPFrame::Purge ()
{
	if (m_treeCtrl)
		m_treeCtrl->Show (FALSE);

	wxProgressDialog* progress = new wxProgressDialog ("Purge des fichiers inutiles",
																		"", m_ensembleCacheIE->NombreFichiers (), this,
																		wxPD_APP_MODAL | wxPD_CAN_ABORT | wxPD_REMAINING_TIME);
	int numeroFichier = 0;
	wxString messageFin;

	while (m_ensembleCacheIE->PurgeSuivant ())
	{
		if (!progress->Update (numeroFichier))
		{
			messageFin = "Tous les fichier n'ont pas t supprims.";
			break;
		}
		numeroFichier++;
	}
	progress->Show (FALSE);
	progress->Destroy ();
	if (!m_ensembleCacheIE->Erreur ().IsEmpty ())
		messageFin = m_ensembleCacheIE->Erreur ();
	if (!messageFin.IsEmpty ())
	{
		wxMessageBox (messageFin, titreApplication, wxICON_EXCLAMATION | wxOK, this);
	}
	GetMenuBar ()->Enable (IECPurge_Purge, FALSE);
	delete m_ensembleCacheIE; m_ensembleCacheIE = NULL;
	if (m_treeCtrl)
	{
		m_treeCtrl->Destroy ();
	  	m_treeCtrl = NULL;
	}
}

void IECPFrame::OnAPropos (wxCommandEvent& event_)
{
	wxMessageBox ("Internet Explorer Cache Purge\n"
					  "\n"
					  "Consultation et suppression des fichiers inutiles\n"
					  "contenus dans le cache d'Internet Explorer.\n"
					  "(dossier \"" + g_nomDossierTIF + "\")\n"
					  "\n"
					  "Copyright  2001, Thierry MARTIN\n"
					  "Ce programme est soumis  la licence GNU GPL", wxString (" propos d'") + titreApplication,
					  wxOK, this);
}

// Dfinition des fonctions membres pour la fentre d'arbre des fichiers

BEGIN_EVENT_TABLE(IECPTreeCtrl, wxTreeCtrl)
	EVT_RIGHT_DOWN(IECPTreeCtrl::OnClicDroit)
	EVT_MENU(IECPurge_CacheSupprimer, IECPTreeCtrl::OnCacheSupprimer)
	EVT_MENU(IECPurge_CacheVoir,		 IECPTreeCtrl::OnCacheVoir)
END_EVENT_TABLE()

IECPTreeCtrl::IECPTreeCtrl (wxWindow* parent_, const wxPoint& position_, const wxSize taille_)
 : wxTreeCtrl (parent_, -1, wxDefaultPosition, wxDefaultSize, wxTR_HAS_BUTTONS | wxSUNKEN_BORDER)
{
	Show (FALSE);
	SetSize (wxRect (position_, taille_));
	wxImageList* listeImages = new wxImageList (16, 16, TRUE);
	listeImages->Add (wxICON (dossier1));
	listeImages->Add (wxICON (dossier2));
	listeImages->Add (wxICON (document));
	SetImageList (listeImages);
	m_baseId = AddRoot ("Temporary Internet File", Image_Dossier);
	SetItemImage (m_baseId, Image_DossierOuvert, wxTreeItemIcon_Expanded);
}

void IECPTreeCtrl::AjoutDossier (const wxString& sousDossier)
{
	m_dossierId = AppendItem (m_baseId, sousDossier, Image_Dossier);
	SetItemImage (m_dossierId, Image_DossierOuvert, wxTreeItemIcon_Expanded);
}

void IECPTreeCtrl::AjoutFichier (const wxString& nomFichier)
{
	AppendItem (m_dossierId, nomFichier, Image_Document);
}

void IECPTreeCtrl::DeveloppeBase ()
{
	Expand (m_baseId);
}

void IECPTreeCtrl::OnCacheSupprimer (wxCommandEvent&)
{
	if (wxMessageBox ("Suppression du fichier " + m_fichierClicNom + " ?", titreApplication, wxICON_QUESTION | wxYES_NO,
							this) == wxYES)
	{
		if (wxRemoveFile (m_fichierClicNom))
			Delete (m_fichierClicId);
		else
			wxMessageBox ("Le fichier " + m_fichierClicNom + " ne peut pas tre supprim.", titreApplication,
							  wxICON_EXCLAMATION | wxOK, this);
	}
}

void IECPTreeCtrl::OnCacheVoir (wxCommandEvent&)
{
	wxExecute ("notepad \"" + m_fichierClicNom + "\"");
}

void IECPTreeCtrl::OnClicDroit (wxMouseEvent& evenement)
{
	int flags;
	wxPoint position = evenement.GetPosition ();
	wxTreeItemId itemId = HitTest (position, flags);
	if ((flags & (wxTREE_HITTEST_ONITEMICON | wxTREE_HITTEST_ONITEMLABEL)) != 0 && GetItemImage (itemId) == Image_Document)
	{
		m_fichierClicNom = g_nomDossierTIF + GetItemText (GetItemParent (itemId)) + '\\' + GetItemText (itemId);
		m_fichierClicId = itemId;
		wxMenu* menu = new wxMenu ();
		menu->Append (IECPurge_CacheVoir, "Voir", "Ouvre le fichier avec le bloc note pour le consulter.");
		menu->Append (IECPurge_CacheSupprimer, "Supprimer", "Supprime seulement ce fichier.");
		PopupMenu (menu, position);
	}
}

wxString IECPTreeCtrl::PremierFichier ()
{
	wxTreeItemIdValue parcoursBase;
	wxTreeItemId dossierId = GetFirstChild (m_baseId, parcoursBase);

	if (!dossierId.IsOk ())
		return "";

	wxTreeItemIdValue parcoursDossier;
	wxTreeItemId fichierId = GetFirstChild (dossierId, parcoursDossier);

	if (!fichierId.IsOk ())
		return "";
	return GetItemText (dossierId) + "\\" + GetItemText (fichierId);
}

bool IECPTreeCtrl::SupprimeFichier (const wxString& nom_)
{
	int position = nom_.Find ('\\');

	if (position == -1) return false;

	wxString nomDossier = nom_.Left (position);
	wxString nomFichier = nom_.Mid (position + 1);

	wxTreeItemIdValue parcoursBase;
	wxTreeItemId dossierId;

	for (dossierId = GetFirstChild (m_baseId, parcoursBase) ; dossierId.IsOk () && nomDossier != GetItemText (dossierId) ;
		  dossierId = GetNextChild (m_baseId, parcoursBase))
	{
	}
	if (!dossierId.IsOk ())
		return false;

	wxTreeItemIdValue parcoursDossier;
	wxTreeItemId fichierId;

	for (fichierId = GetFirstChild (dossierId, parcoursDossier) ; fichierId.IsOk () && nomFichier != GetItemText (fichierId) ;
		  fichierId = GetNextChild (dossierId, parcoursDossier))
	{
	}
	if (!fichierId.IsOk ())
		return false;

	Delete (fichierId);
	if (!GetFirstChild (dossierId, parcoursDossier).IsOk ())
	{
		Delete (dossierId);
	}
	return true;
}

// Dfinition des fonctions membres pour l'application

IMPLEMENT_APP(IECPApp)

IECPApp::IECPApp ()
 : m_purgeAuto (false)
{
}

bool IECPApp::OnCmdLineParsed (wxCmdLineParser& parser_)
{
	if (!wxApp::OnCmdLineParsed (parser_))
		return false;
	m_purgeAuto = parser_.Found ("purge");
	return true;
}

bool IECPApp::OnInit ()
{
   wxCmdLineParser parser (argc, argv);
   bool cont;

   OnInitCmdLine (parser);
   switch (parser.Parse (false))
   {
	case -1:
		cont = OnCmdLineHelp(parser);
		break;

	case 0:
		cont = OnCmdLineParsed(parser);
		break;

	default:
		cont = OnCmdLineError(parser);
		break;
   }
	if (!cont)
		return false;

	if (m_purgeAuto)
		return PurgeAuto ();

	IECPFrame* frame = new IECPFrame (titreApplication, wxPoint (-1, -1), wxSize (450, 450));

	frame->Show (TRUE);
	SetTopWindow (frame);
	return TRUE;
}

void IECPApp::OnInitCmdLine (wxCmdLineParser& parser_)
{
	static const wxCmdLineEntryDesc cmdLineDesc [] =
	{
		{
			wxCMD_LINE_SWITCH,
			wxEmptyString,
			"purge",
			"Recherche puis purge les fichiers sans confirmation.",
			wxCMD_LINE_VAL_NONE,
			0x0
		},
		{
			wxCMD_LINE_NONE,
			wxEmptyString,
			wxEmptyString,
			wxEmptyString,
			wxCMD_LINE_VAL_NONE,
			0x0
		}
	};
	parser_.SetDesc (cmdLineDesc);
	wxApp::OnInitCmdLine (parser_);
}

bool IECPApp::PurgeAuto ()
{
	TTableauCacheIEAPurger aPurger;
	TEnsembleCacheIE ensembleCacheIE (aPurger);
	wxProgressDialog* progress;

	if (!ensembleCacheIE.Erreur ().IsEmpty ())
	{
		wxMessageBox (ensembleCacheIE.Erreur (), titreApplication, wxOK | wxICON_EXCLAMATION, NULL);
		return false;
	}
	progress = new wxProgressDialog ("Recherche des fichiers inutiles",
												"Parcours du cache Internet Explorer", 0, NULL, wxPD_APP_MODAL | wxPD_CAN_ABORT);
	while (ensembleCacheIE.RechercheSuivant ())
	{
		if (!progress->Update (-1))
		{
			progress->Show (FALSE);
			progress->Destroy ();
			return true;
		}
	}
	progress->Show (FALSE);
	progress->Destroy ();
	if (!ensembleCacheIE.Erreur ().IsEmpty ())
	{
		wxMessageBox (ensembleCacheIE.Erreur (), titreApplication, wxOK | wxICON_EXCLAMATION, NULL);
		return false;
	}
	if (!ensembleCacheIE.NombreFichiers ())
	{	// Rien  purger
		return false;
	}
/*
	wxString etat;

	etat = wxString::Format ("%u fichier", m_ensembleCacheIE->NombreFichiers ());
	if (m_ensembleCacheIE->NombreFichiers () > 1) etat += "s";
	etat += wxString::Format (" pour %lld octet", m_ensembleCacheIE->TailleTotale ());
	if (m_ensembleCacheIE->TailleTotale () > 1)
		etat += "s, ils peuvent tre purgs";
	else
		etat += ", il peut tre purg";
	etat += " (Fichier/Purger)."; */

	progress = new wxProgressDialog ("Purge des fichiers inutiles",
												"", ensembleCacheIE.NombreFichiers (), NULL, wxPD_APP_MODAL | wxPD_CAN_ABORT | wxPD_REMAINING_TIME);

	int numeroFichier = 0;

	while (ensembleCacheIE.PurgeSuivant ())
	{
		if (!progress->Update (numeroFichier))
		{
			return true;
		}
		numeroFichier++;
	}
	progress->Show (FALSE);
	progress->Destroy ();
	if (!ensembleCacheIE.Erreur ().IsEmpty ())
	{
		wxMessageBox (ensembleCacheIE.Erreur (), titreApplication, wxICON_EXCLAMATION | wxOK, NULL);
	}
	return false;
}

// vim:ts=3:sw=3:
